"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_intent_1 = __importDefault(require("./base-intent"));
const chalk_1 = __importDefault(require("chalk"));
const editorinterface_save_1 = require("../action/editorinterface-save");
const sidebarwidget_add_1 = require("../action/sidebarwidget-add");
class SidebarWidgetAddIntent extends base_intent_1.default {
    isSidebarUpdate() {
        return true;
    }
    isEditorInterfaceIntent() {
        return true;
    }
    isGroupable() {
        return true;
    }
    groupsWith(other) {
        return other.isGroupable() && other.isEditorInterfaceIntent() && this.isSameContentType(other);
    }
    endsGroup() {
        return false;
    }
    shouldSave() {
        return false;
    }
    shouldPublish() {
        return false;
    }
    toActions() {
        return [
            new sidebarwidget_add_1.SidebarWidgetAddAction(this.payload.contentTypeId, this.payload.sidebarWidget.widgetId, this.payload.sidebarWidget.widgetNamespace, this.payload.sidebarWidget.insertBeforeWidgetId, this.payload.sidebarWidget.settings),
            new editorinterface_save_1.SaveEditorInterfaceAction(this.payload.contentTypeId)
        ];
    }
    toPlanMessage() {
        const { settings, widgetId, widgetNamespace } = this.payload.sidebarWidget;
        const settingDetails = Object.keys(settings).map((settingName) => (0, chalk_1.default) `{italic ${settingName}}: "${settings[settingName].toString()}"`);
        const createDetails = [(0, chalk_1.default) `{italic widgetNamespace}: "${widgetNamespace}"`, ...settingDetails];
        return {
            heading: (0, chalk_1.default) `Update sidebar for Content Type {bold.yellow ${this.getContentTypeId()}}`,
            details: [],
            sections: [
                {
                    heading: (0, chalk_1.default) `Add sidebar widget {yellow ${widgetId}}`,
                    details: createDetails
                }
            ]
        };
    }
}
exports.default = SidebarWidgetAddIntent;
//# sourceMappingURL=sidebarwidget-add.js.map