"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = __importStar(require("joi"));
const allowed_resources_schema_1 = require("./allowed-resources-schema");
const validation = (name, constraint) => Joi.object({
    [name]: constraint,
    message: Joi.string().allow(null)
});
const range = (type) => Joi.object({
    min: Joi[type]().allow(null),
    max: Joi[type]().allow(null)
});
const rangeForDate = () => Joi.object({
    before: Joi.string().optional().allow(null),
    after: Joi.string().optional().allow(null),
    min: Joi.string().optional().allow(null),
    max: Joi.string().optional().allow(null)
});
const linkContentType = validation('linkContentType', Joi.array().items(Joi.string()));
const inValidation = validation('in', Joi.array());
const linkMimetypeGroup = validation('linkMimetypeGroup', Joi.array().items(Joi.string()));
const size = validation('size', range('number'));
const rangeValidation = validation('range', range('number'));
const regexp = validation('regexp', Joi.object({
    pattern: Joi.string(),
    flags: Joi.string().allow(null).optional()
}));
const prohibitRegexp = validation('prohibitRegexp', Joi.object({
    pattern: Joi.string(),
    flags: Joi.string().allow(null).optional()
}));
const unique = validation('unique', Joi.boolean());
const dateRange = validation('dateRange', rangeForDate());
const assetImageDimensions = validation('assetImageDimensions', Joi.object({
    width: range('number'),
    height: range('number')
}));
const assetFileSize = validation('assetFileSize', range('number'));
const nodes = validation('nodes', Joi.object({
    'embedded-entry-block': Joi.array(),
    'embedded-entry-inline': Joi.array(),
    'embedded-asset-block': Joi.array(),
    'embedded-resource-block': Joi.object({
        validations: Joi.array().items(size),
        allowedResources: allowed_resources_schema_1.allowedResourcesSchema
    }),
    'entry-hyperlink': Joi.array(),
    'asset-hyperlink': Joi.array(),
    hyperlink: Joi.array()
}));
const enabledMarks = validation('enabledMarks', Joi.array().items(Joi.string().valid('bold', 'italic', 'code', 'underline', 'superscript', 'subscript')));
const enabledNodeTypes = validation('enabledNodeTypes', Joi.array().items(Joi.string().valid('heading-1', 'heading-2', 'heading-3', 'heading-4', 'heading-5', 'heading-6', 'ordered-list', 'unordered-list', 'hr', 'blockquote', 'table', 'embedded-entry-block', 'embedded-asset-block', 'embedded-resource-block', 'hyperlink', 'entry-hyperlink', 'asset-hyperlink', 'embedded-entry-inline')));
const fieldValidations = Joi.alternatives().try(linkContentType, inValidation, linkMimetypeGroup, size, rangeValidation, regexp, prohibitRegexp, unique, dateRange, assetImageDimensions, assetFileSize, nodes, enabledMarks, enabledNodeTypes);
exports.default = fieldValidations;
//# sourceMappingURL=field-validations-schema.js.map